@@ -7,24 +7,12 @@ from os.path import isfile
 from os.path import join
 from os.path import split
 from re import sub as re_sub
-from sqlite3 import Connection
 from typing import Dict
 from typing import List
 from typing import Optional
-from typing import Tuple
 
-from falocalrepo_database import connect_database
-from falocalrepo_database import journals_indexes
-from falocalrepo_database import journals_table
-from falocalrepo_database import read_setting
-from falocalrepo_database import search_journals as db_search_journals
-from falocalrepo_database import search_submissions as db_search_submissions
-from falocalrepo_database import select
-from falocalrepo_database import select_all
-from falocalrepo_database import submissions_indexes
-from falocalrepo_database import submissions_table
+from falocalrepo_database import FADatabase
 from falocalrepo_database import tiered_path
-from falocalrepo_database import users_indexes
 from flask import Flask
 from flask import abort
 from flask import redirect
@@ -41,9 +29,9 @@ last_search: dict = {
     "table": "",
     "order": [],
     "params": {},
-    "results": [],
-    "indexes": {}
+    "results": []
 }
+db_path: str = "FA.db"
 
 
 def clean_username(username: str) -> str:
@@ -56,7 +44,7 @@ def favicon():
 
 
 @app.errorhandler(404)
-def not_found(err: NotFound):
+def not_found(_err: NotFound):
     return render_template(
         "not_found.html",
         title=f"{app.name} · Content not Found"
@@ -65,12 +53,17 @@ def not_found(err: NotFound):
 
 @app.route("/")
 def root():
-    db_temp: Connection = connect_database("FA.db")
-    sub_n: int = int(read_setting(db_temp, "SUBN"))
-    jrn_n: int = int(read_setting(db_temp, "JRNN"))
-    usr_n: int = int(read_setting(db_temp, "USRN"))
-    version: str = read_setting(db_temp, "VERSION")
-    db_temp.close()
+    global db_path
+
+    sub_n: int
+    jrn_n: int
+    usr_n: int
+    version: str
+    with FADatabase(db_path) as db:
+        sub_n = int(db.settings["SUBN"])
+        jrn_n = int(db.settings["JRNN"])
+        usr_n = int(db.settings["USRN"])
+        version = db.settings["VERSION"]
 
     return render_template(
         "root.html",
@@ -84,21 +77,24 @@ def root():
 
 @app.route("/user/<username>")
 def user(username: str):
-    db_temp: Connection = connect_database("FA.db")
-    entry: Optional[tuple] = select(db_temp, "USERS", ["*"], ["USERNAME"], [clean_username(username)]).fetchone()
+    if username != (username_clean := clean_username(username)):
+        return redirect(f"/user/{username_clean}")
 
-    if entry is None:
-        db_temp.close()
+    global db_path
+
+    user_entry: Optional[dict]
+    with FADatabase(db_path) as db:
+        user_entry = db.users[username]
+
+    if user_entry is None:
         return abort(404)
 
-    folders = entry[users_indexes["FOLDERS"]]
-    gallery = entry[users_indexes["GALLERY"]]
-    scraps = entry[users_indexes["SCRAPS"]]
-    favorites = entry[users_indexes["FAVORITES"]]
-    mentions = entry[users_indexes["MENTIONS"]]
-    journals = entry[users_indexes["JOURNALS"]]
-
-    db_temp.close()
+    folders: List[int] = list(filter(bool, user_entry["FOLDERS"].split(",")))
+    gallery: List[int] = list(map(int, filter(bool, user_entry["GALLERY"].split(","))))
+    scraps: List[int] = list(map(int, filter(bool, user_entry["SCRAPS"].split(","))))
+    favorites: List[int] = list(map(int, filter(bool, user_entry["FAVORITES"].split(","))))
+    mentions: List[int] = list(map(int, filter(bool, user_entry["MENTIONS"].split(","))))
+    journals: List[int] = list(map(int, filter(bool, user_entry["JOURNALS"].split(","))))
 
     return render_template(
         "user.html",
@@ -142,17 +138,20 @@ def search_user_journals(username: str):
 
 def search(table: str):
     global last_search
+    global db_path
 
     if request.args:
         params: Dict[str, List[str]] = dict(map(
             lambda kv: (kv[0], json_loads(kv[1])),
             request.args.items()
         ))
+        order: List[str] = params.get("order", ["ID DESC"])
         limit: int = 50
         offset: int = params.get("offset", 0)
         offset = 0 if offset < 0 else offset
 
-        params["order"] = params.get("order", ["ID DESC"])
+        if "order" in params:
+            del params["order"]
         if "limit" in params:
             del params["limit"]
         if "offset" in params:
@@ -161,31 +160,32 @@ def search(table: str):
         if (last_search["table"], last_search["params"]) != (table, params):
             last_search["table"] = table
             last_search["params"] = deepcopy(params)
-            db_temp: Connection = connect_database("FA.db")
-            if table == "submissions":
-                if list(params.keys()) == ["order"]:
-                    last_search["results"] = select_all(db_temp, submissions_table, ["*"], params["order"]).fetchall()
-                else:
-                    last_search["results"] = db_search_submissions(db_temp, **params)
-                last_search["indexes"] = submissions_indexes
-            elif table == "journals":
-                if list(params.keys()) == ["order"]:
-                    last_search["results"] = select_all(db_temp, journals_table, ["*"], params["order"]).fetchall()
-                else:
-                    last_search["results"] = db_search_journals(db_temp, **params)
-                last_search["indexes"] = journals_indexes
-            db_temp.close()
+            if "author" in params:
+                params["replace(author, '_', '')"] = list(map(clean_username, params["author"]))
+                del params["author"]
+            with FADatabase(db_path) as db:
+                if table == "submissions":
+                    if list(params.keys()) == ["order"]:
+                        last_search["results"] = list(db.submissions)
+                    else:
+                        last_search["results"] = list(db.submissions.cursor_to_dict(
+                            db.submissions.select(params, like=True, order=order)))
+                elif table == "journals":
+                    if list(params.keys()) == ["order"]:
+                        last_search["results"] = list(db.journals)
+                    else:
+                        last_search["results"] = list(db.journals.cursor_to_dict(
+                            db.journals.select(params, like=True, order=order)))
 
         return render_template(
             "search_results.html",
             title=f"{app.name} · {table.title()} Search Results",
             table=table,
-            params=params,
+            params=last_search["params"],
             limit=limit,
             offset=offset,
             results=last_search["results"][offset:offset + limit],
-            results_total=len(last_search["results"]),
-            indexes=last_search["indexes"]
+            results_total=len(last_search["results"])
         )
     else:
         return render_template(
@@ -196,33 +196,21 @@ def search(table: str):
         )
 
 
-@app.route("/submission/<int:id_>/file/")
-def submission_file(id_: int):
-    db_temp: Connection = connect_database("FA.db")
-    sub_dir: str = join(read_setting(db_temp, "FILESFOLDER"), *split(tiered_path(id_)))
-    sub_ext: Optional[Tuple[str]] = select(db_temp, "SUBMISSIONS", ["FILEEXT"], ["ID"], [id_]).fetchone()
-    db_temp.close()
-
-    if isfile(path := join(sub_dir, f"submission.{sub_ext[0]}")):
-        return send_file(path)
-    else:
-        return abort(404)
-
-
 @app.route("/journal/<int:id_>/")
 def journal(id_: int):
-    db_temp: Connection = connect_database("FA.db")
-    jrnl: Optional[tuple] = select(db_temp, "JOURNALS", ["*"], ["ID"], [id_]).fetchone()
-    db_temp.close()
+    global db_path
+
+    jrnl: Optional[dict]
+    with FADatabase(db_path) as db:
+        jrnl = db.journals[id_]
 
     if jrnl is None:
         return abort(404)
 
     return render_template(
         "journal.html",
-        title=f"{app.name} · {jrnl[journals_indexes['TITLE']]} by {jrnl[journals_indexes['AUTHOR']]}",
-        journal=jrnl,
-        indexes=journals_indexes
+        title=f"{app.name} · {jrnl['TITLE']} by {jrnl['AUTHOR']}",
+        journal=jrnl
     )
 
 
@@ -233,28 +221,52 @@ def submission_view(id_: int):
 
 @app.route("/submission/<int:id_>/")
 def submission(id_: int):
-    db_temp: Connection = connect_database("FA.db")
-    sub: Optional[tuple] = select(db_temp, "SUBMISSIONS", ["*"], ["ID"], [id_]).fetchone()
-    db_temp.close()
+    global db_path
+
+    sub: Optional[dict]
+    with FADatabase(db_path) as db:
+        sub: Optional[dict] = db.submissions[id_]
 
     if sub is None:
         return abort(404)
 
     file_type: Optional[str] = ""
-    if (ext := sub[submissions_indexes['FILEEXT']]) in ("jpg", "jpeg", "png", "gif"):
+    if (ext := sub["FILEEXT"]) in ("jpg", "jpeg", "png", "gif"):
         file_type = "image"
+    elif ext == "mp3":
+        file_type = "audio"
     elif not ext:
         file_type = None
 
     return render_template(
         "submission.html",
-        title=f"{app.name} · {sub[submissions_indexes['TITLE']]} by {sub[submissions_indexes['AUTHOR']]}",
-        sub_id=id_,
+        title=f"{app.name} · {sub['TITLE']} by {sub['AUTHOR']}",
         submission=sub,
-        file_type=file_type,
-        indexes=submissions_indexes
+        file_type=file_type
     )
 
 
-def server(host: str = "0.0.0.0", port: int = 8080):
+@app.route("/submission/<int:id_>/file/")
+def submission_file(id_: int):
+    global db_path
+
+    sub: Optional[dict]
+    sub_dir: str
+    with FADatabase(db_path) as db:
+        sub = db.submissions[id_]
+        sub_dir = join(dirname(db_path), db.settings["FILESFOLDER"], *split(tiered_path(id_)))
+
+    if sub is None:
+        return abort(404)
+    elif isfile(path := join(sub_dir, f"submission.{sub['FILEEXT']}")):
+        return send_file(path)
+    else:
+        return abort(404)
+
+
+def server(database_path: str, host: str = "0.0.0.0", port: int = 8080):
+    global db_path
+
+    db_path = abspath(database_path)
+
     app.run(host=host, port=port)
